Unit Ensam;{ Ensamblador }
Interface
  Uses General, InsGame, AnaSin;

  Procedure Ensamblador;

Implementation

  Type

    Operations = Set of OperationPart;

    AssemblyTable = array[1..MaxLabel] of LongInt;

  var
    NoArguments,
    OneArgument,
    TwoArguments,
    FourArguments,
    Jumps,                          (* salto *)
    Patterns,                       (* activacin de procesadores *)
    IntInstr,                       (* relacionales y de desplazamiento *)
    loops
                   : Operations;
    Op : OperationPart;
    Yo : Longint;
    Addr : Longint;
    Arg1, Arg2, Arg3, Arg4, Arg5 : LongInt;
    Table : AssemblyTable;
    Flis : text;


  Procedure LineaTexto(var linea : string);
  var
    i : word;
    ch : char;
  begin
    read(finput, ch);
    linea := '';
    while Not eof(finput) and (ch <> #13 ) do
      begin
        linea := linea + ch;
        read(finput, ch);
      end;
    if ch = #13 then read( finput, ch);
  end;

  Procedure NextInstruction;
  begin
    read(temp2,Yo);
    Op := OperationPart(Yo);
    if Op in NoArguments then { SKIP }
    else if Op in OneArgument then read(temp2, Arg1)
    else if Op in TwoArguments then
      begin
        read(temp2, Arg1);
        read(temp2, Arg2)
      end
    else if Op in FourArguments then
      begin
        read(temp2,Arg1);
        read(temp2,Arg2);
        read(temp2,Arg3);
        read(temp2,Arg4)
      end
  end;

  Procedure Emit1(Op: OperationPart);
  begin
    if Emitting then writeln( Flis, addr:44,':':2, nc[Op]);
    Emit( fcode, ord(Op) );
    addr := addr + 1
  end;

  Procedure Emit2(Op: OperationPart; Arg: LongInt);
  var texto :string;
  begin
   if Emitting and ( Op = newline2 ) then
      begin
        LineaTexto(texto);
        writeln(Flis, Arg:4 ,':':2, texto);
      end
    else if Emitting and (Op <> newline2 ) then
      writeln( Flis, addr:44,':':2, nc[Op], ' ', Arg );
    if Op <> newline2 then
      begin
        Emit( fcode, ord(Op));
        Emit( fcode, Arg);
        addr := addr + 2
      end;
  end;

  Procedure Emit3(Op: OperationPart; Arg1, Arg2: LongInt);
  begin
    if Emitting then
      writeln( Flis, addr:44,':':2, nc[Op],' ', Arg1,' ', Arg2);
    Emit( fcode, ord(Op));
    Emit( fcode, Arg1);
    Emit( fcode, Arg2);
    addr := addr + 3
  end;

  Procedure Emit5(Op: OperationPart; Arg1,Arg2,Arg3,Arg4: LongInt);
  begin
    if Emitting then
      writeln( Flis, addr:44,':':2,
      nc[Op],' ', Arg1, ' ',Arg2, ' ',Arg3, ' ', Arg4);
    Emit( fcode, ord(Op));
    Emit( fcode, Arg1);
    Emit( fcode, Arg2);
    Emit( fcode, Arg3);
    Emit( fcode, Arg4);
    addr := addr + 5
  end;


  function Optimize( specialcase: boolean) : boolean;
  begin
    Optimize := specialcase
  end;

  function Jumpdispl( LabelNo: Longint): Longint;
  begin
    Jumpdispl := Table[LabelNo] - addr
  end;

  Procedure assignx( Length: Longint);
  begin
    if Optimize( length = 1) then
      Emit1( simpleassign2 )
    else
      Emit2( assign2, length);
    NextInstruction
  end;

  Procedure PrivateAssign( length: Longint);
  begin
    if Optimize( length = 1) then
      Emit1( PrivateSimpleAssign2 )
    else
      Emit2( PrivateAssign2, length);
    NextInstruction
  end;

  Procedure pushconstant( value : Longint );
  begin
    Emit2( constant2, table[value] );
    NextInstruction
  end;

  procedure Programx( VarLabel, TempControlMax, TempElementalMax,
                      BeginLabel: Longint);
  begin
    { OP IN [PROCEDURE2,PROGRAM2] }
    Emit5( Program2,
           Table[VarLabel],
           Table[TempControlMax],
           Table[TempElementalMax],
           JumpDispl(BeginLabel) );
    NextInstruction
  end;

  Procedure Procedurex( VarLabel, TempControl, TempElemental,
                        BeginLabel : Longint);
  begin
    Emit5( Procedure2,
           Table[VarLabel],
           Table[TempControl],
	   Table[TempElemental],
           JumpDispl(BeginLabel) );
    NextInstruction
  end;

  Procedure DefAddr( LabelNo: Longint);
  begin
    Table[LabelNo] := addr;
    NextInstruction
  end;

  Procedure DefArg( LabelNo, Value: Longint);
  begin
    Table[LabelNo] := value;
    NextInstruction
  end;

  Procedure Field( Displ: Longint);
  begin
    if Optimize( displ = 0 ) then { EMPTY }
    else  Emit2( Field2, displ);
    NextInstruction
  end;

  Procedure Jump( Op: OperationPart; LabelNo: Longint);
  begin
    { OP IN [DO2, GOTO2, NOTDO, GOIFZERO, GOIFNOZERO,
      INITFOR, INITNESTEDFOR]
    }
    Emit2( Op, Jumpdispl(LabelNo));
    NextInstruction
  end;

  Procedure Patternx( Op: OperationPart; LabelNo, ThresholdValue : Longint);
  begin
    Emit3( Op, Jumpdispl(LabelNo), ThresholdValue);
    NextInstruction
  end;

  Procedure Loopsx( Op: OperationPart; LabelNo, StepValue : Longint);
  begin
    Emit3( Op, Jumpdispl(LabelNo), StepValue);
    NextInstruction
  end;

  Procedure IntIntrx( Op : OperationPart; Length : Longint );
  begin
    if Optimize( Length = 1) then
      case Op of
        and2        : Emit1( AndInt2);
        equal2      : Emit1( EqualInt2);
        greater2    : Emit1( GreaterInt2);
        less2       : Emit1( LessInt2);
        not2        : Emit1( NotInt2);
        notequal2   : Emit1( NotEqualInt2);
        notgreater2 : Emit1( NotGreaterInt2);
        notless2    : Emit1( NotLessInt2);
        or2         : Emit1( OrInt2);
        shl2        : Emit1( ShlInt2);
        shr2        : Emit1( ShrInt2);
        xor2        : Emit1( XorInt2);
      end (* case *)
    else Emit2( Op, Length);
    NextInstruction
  end;


  Procedure ProcCall( Level, LabelNo: Longint);
  var Displ: Longint;
  begin
    Displ := Jumpdispl(LabelNo);
    if Optimize( Level = 0) then Emit2( localcall2, displ)
    else if Optimize( Level = 1) then Emit2( globalcall2, displ)
    else if Optimize( Level = 2) then Emit2( globalcalltwo2, displ)
    else if Optimize( Level = 3) then Emit2( globalcallthree2, displ)
    else  Emit3( proccall2, level, displ);
    NextInstruction
  end;

  Procedure Value( Length: Longint);
  begin
    if Optimize( Length = 1) then Emit1( simplevalue2)
    else Emit2( value2, Length);
    NextInstruction
  end;

  Procedure Variable( Level, Displ: Longint);
  begin
    NextInstruction;
    while Optimize( Op = Field2) do
      begin
        displ := displ + Arg1;
        NextInstruction
      end;

    if Optimize( Level = 0) then
      if ( Op = Value2) and ( Arg1 = 1) then
        begin
          Emit2( localvalue2, displ);
          NextInstruction
        end
      else Emit2( localvar2, displ)
    else if Optimize( Level = 1) then
      if ( Op = Value2) and ( Arg1 = 1) then
        begin
          Emit2( globalvalue2, displ);
          NextInstruction
        end
      else Emit2( globalvar2, displ)
    else if Optimize( Level = 2) then
      if ( Op = Value2) and ( Arg1 = 1) then
        begin
          Emit2( globalvaluetwo2, displ);
          NextInstruction
        end
      else Emit2( globalvartwo2, displ)
    else if Optimize( Level = 3) then
      if ( Op = Value2) and ( Arg1 = 1) then
        begin
          Emit2( globalvaluethree2, displ);
          NextInstruction
        end
      else Emit2( globalvarthree2, displ)
    else Emit3(variable2, level, displ)
  end;

  Procedure copyInstruction;
  begin
    if Op in NoArguments then
      Emit1(Op)
    else if Op in OneArgument then
      Emit2(Op, Arg1)
    else if Op in TwoArguments then
      Emit3(Op, Arg1, Arg2)
    else { Op in fourArguments }
      Emit5(Op, Arg1, Arg2, Arg3, Arg4);
    NextInstruction
  end;

  Procedure Assemble;
  begin
    addr := 0;
    NextInstruction;
    while Op <> endprog2 do
      if Op = assign2 then
	assignx(Arg1)
      else if Op = PrivateAssign2 then
        PrivateAssign( Arg1)
      else if Op = pushconst2 then
	pushconstant( Arg1 )
      else if Op = Program2 then
	Programx(Arg1, Arg2, Arg3, Arg4)
      else if Op = Procedure2 then
	Procedurex( Arg1, Arg2, Arg3, Arg4)
      else if Op = defAddr2 then
	defAddr(Arg1)
      else if Op = defArg2 then
	defArg(Arg1, Arg2)
      else if Op = field2 then
	field(Arg1)
      else if Op in Jumps then
	Jump(Op, Arg1)
      else if Op in Patterns then
        Patternx( Op, Arg1, Arg2)
      else if Op in Loops then
        Loopsx( Op, Arg1, Arg2)
      else if Op in IntInstr then
        IntIntrx( Op, Arg1)
      else if Op = proccall2 then
	proccall( Arg1, Arg2)
      else if Op = value2 then
	value(Arg1)
      else if Op = variable2 then
	variable( Arg1, Arg2)
      else
	copyInstruction;
     Emit1(endprog2)
  end;

  Procedure initialize;
  var LabelNo: Longint;
  begin
    NoArguments := [ active2, add2,
                     complement2, CopyNoTops2, copyTops2,
                     debug2, divide2,
                     endBlock2, endNestedPattern2, endPattern2, endprog2,
                     endSinglePattern2, endRelax2,
                     getTime2,
		     max2, min2, minus2, modulo2, multiply2,
                     privategettime2,
                     privatesimpleassign2,
                     privatesettime2,
                     privatereadint2,
                     pushcounter2, pushpattern2,popPattern2,
                     setTime2, size2, subtract2,
		     randomize2, readint2, readlnx2, relax2,
		     writeint2];

    OneArgument := [ and2, assign2,
                     broadcast2,
                     constant2, equal2,
		     less2, greater2, fillzero2,
		     Notdo2, goifzero2, goifNozero2,
		     do2, endproc2, field2,
		     in2, initFor2, initNestedFor2,
                     Not2, Notequal2, Notgreater2, Notless2,
		     newline2,
		     or2, free2, shl2,
                     privateassign2, 
                     pushname2, pushconst2,
		     random2,
                     segbroadcast2, shared2,
		     shr2, singleControl2, singlePattern2,
		     xor2, goto2, value2, defAddr2, newblock2];

    TwoArguments := [ loop2,
                      nestedLoop2,
                      nestedpattern2,
                      privatesegassign2,
                      pattern2, proccall2,
                      segassign2, segdyninit2, segindex2,
                      variable2, varparam2,
		      writeStr2,
                      defArg2];

    FourArguments := [ fill2,
                       index2,
                       marked2,
                       seginit2,
                       program2,
                       procedure2 ];
    Jumps := [ do2,
               goto2, goifzero2, goifNozero2,
               initFor2, initNestedFor2,
               Notdo2  ];
    loops := [ loop2,
               nestedLoop2];
    Patterns := [ NestedPattern2,
                  Pattern2 ];
    IntInstr := [ and2,
                  equal2,
                  greater2,
                  less2,
                  not2,
                  notequal2,
                  notgreater2,
                  notless2,
                  or2,
                  shl2, shr2,
                  xor2 ];

    LabelNo := 1;
    while LabelNo <= MaxLabel do
      begin
	Table[LabelNo] := 0;
	LabelNo := LabelNo +1
      end;
  end;

  Procedure Ensamblador;
  begin
    reset(finput);
    assign( Flis , code + '.lis');
    rewrite(Flis);
    initialize;
    Assemble;
    rerun;
    Assemble;
    close(Flis);
    close(finput);
  end;

end.
