--{{{  !!!! READ.ME !!!!
-- Para buscar partes de codigo que se pueden ver afectadas al incrementar el
-- tamanio del arbol buscar la palabra WWAARRNNIINNGG en todos los ficheros
-- *.OCC y comprobar en esos puntos.
--}}}  

#INCLUDE "core.inc"

--================= MAIN CONSTANTS ===========================================
VAL INT x.trans             IS 31 :   -- Total number of OPERATIVE transputers used
VAL INT max.procs.per.trans IS 512 : -- MUST BE power of 2. Maximum number of PRAM processor on each slave transputer
VAL INT leaf.nodes          IS 18 :    -- Number of leaf nodes in the tree structure
--                        [0        , 1,         2,         3,         4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30] :
VAL [x.trans]INT width IS [max.width, max.width, max.width, max.width, 2, 2, 2, 2, 2, 2,  2,  2,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0] :
--============================================================================
VAL INT max.processor       IS max.procs.per.trans - 1 :
--{{{  COMMENT to max.processor
-- Numero de procesadores PRAM en cada transputer no-root.
-- max.procs.per.trans es el numero total de procesadores, pero uno de ellos
-- realiza la funcion de control local.
--}}}  

--{{{  MEMORY CONSTANTS
--{{{  COMMENT Memory Distribution
--  Capa     P.E.   Celdas    Total       Bytes
--     1     Ctrl   240000   240000     960 000
--     1       32      128     4096      16 384
--     2       64      128     8192      32 768
--     3      128      512    65536     262 144
--     4      512     1024   524288   2 097 152
--  TOTAL:              241792   842112   3 368 448
--  Mb                                            3
--  3 Mb =                                3 145 728
--
--}}}  


VAL [x.trans]INT CONTROL.MEMORY.SIZE IS [240000, 512, 512, 512,
				  512, 512, 512, 512, 512, 512, 512, 512, 512,
				  8192, 8192, 8192, 8192, 8192, 8192,
				  8192, 8192, 8192, 8192, 8192, 8192,
				  8192, 8192, 8192, 8192, 8192, 8192] :

VAL [x.trans]INT L.STACK.SIZE IS [128, 128, 128, 128,
				  512, 512, 512, 512, 512, 512, 512, 512, 512,
				  1133, 1133, 1133, 1133, 1133, 1133,
				  1133, 1133, 1133, 1133, 1133, 1133,
				  1133, 1133, 1133, 1133, 1133, 1133] :

VAL INT M.BLOCK.1           IS 244096 :
VAL INT M.BLOCK.2           IS 252672 :
VAL INT M.BLOCK.3           IS 285440 :
VAL INT M.BLOCK.4           IS 872595 :
VAL INT MAX.M.BLOCK         IS M.BLOCK.4 :  -- The bigger of the former values

-- If M.BLOCK constants are changed, it is necessary to make also a change
-- in ROOT.OCC and WORKER.OCC files for the size of the Memory array.

--{{{  Division threshold
VAL [x.trans]INT division.threshold IS [32,
		 63, 63, 63,
		 63, 63, 63,
		 63, 63, 63,
		 63, 63, 63,
		 max.processor, max.processor, max.processor,
		 max.processor, max.processor, max.processor,
		 max.processor, max.processor, max.processor,
		 max.processor, max.processor, max.processor,
		 max.processor, max.processor, max.processor,
		 max.processor, max.processor, max.processor] :

-- WARNING: Except for the root transputer, the value for division.threshold
--          must be one less than the number of PRAM processors on that
--          transputer. This is so because there is a control processor on
--          each transputer, consuming so much memory as any other processor.
--          In the root transputer, the control processor has its own memory.

-- WARNING: the division threshold is supposed to be the same in each level of the tree (see the first.PRAM.id[] definition)
-- WARNING: The elements of division.threshold must be <= max.processor
--{{{  COMMENT to division.threshold
-- division.threshold

-- Is the maximum number of PRAM processors simulated in each transputer.
-- This value is taken into account when the processors activation is carried
-- on: If the load is below this value, the work is retained in the transputer.
-- If the load is bigger than this value, the work must be divided among the
-- children transputers (If possible).

-- In leaf nodes the threshold is max.processor because there are
-- max.procs.per.trans processors, but one of them supports control tasks, so
-- the number of available processors is: max.procs.per.trans-1=max.processor
--}}}  
--}}}  

VAL [x.trans]INT C.REGISTER.SIZE IS [1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,
				     16, 16, 16, 16, 16, 16, 16, 16, 16,
				     16, 16, 16, 16, 16, 16, 16, 16, 16] :
VAL MAX.C.REGISTER.SIZE IS 16 :


VAL [x.trans]INT MEMORY.BASE IS[0, M.BLOCK.1, M.BLOCK.1, M.BLOCK.1,
				   M.BLOCK.2, M.BLOCK.2, M.BLOCK.2,
				   M.BLOCK.2, M.BLOCK.2, M.BLOCK.2,
				   M.BLOCK.2, M.BLOCK.2, M.BLOCK.2,
				   M.BLOCK.3, M.BLOCK.3, M.BLOCK.3,
				   M.BLOCK.3, M.BLOCK.3, M.BLOCK.3,
				   M.BLOCK.3, M.BLOCK.3, M.BLOCK.3,
				   M.BLOCK.3, M.BLOCK.3, M.BLOCK.3,
				   M.BLOCK.3, M.BLOCK.3, M.BLOCK.3,
				   M.BLOCK.3, M.BLOCK.3, M.BLOCK.3] :

VAL [x.trans]INT MEMORY.TOP IS[M.BLOCK.1 - 1,
				M.BLOCK.2 - 1, M.BLOCK.2 - 1, M.BLOCK.2 - 1,
				M.BLOCK.3 - 1, M.BLOCK.3 - 1, M.BLOCK.3 - 1,
				M.BLOCK.3 - 1, M.BLOCK.3 - 1, M.BLOCK.3 - 1,
				M.BLOCK.3 - 1, M.BLOCK.3 - 1, M.BLOCK.3 - 1,
				M.BLOCK.4 - 1, M.BLOCK.4 - 1, M.BLOCK.4 - 1,
				M.BLOCK.4 - 1, M.BLOCK.4 - 1, M.BLOCK.4 - 1,
				M.BLOCK.4 - 1, M.BLOCK.4 - 1, M.BLOCK.4 - 1,
				M.BLOCK.4 - 1, M.BLOCK.4 - 1, M.BLOCK.4 - 1,
				M.BLOCK.4 - 1, M.BLOCK.4 - 1, M.BLOCK.4 - 1,
				M.BLOCK.4 - 1, M.BLOCK.4 - 1, M.BLOCK.4 - 1] :
--}}}  

--{{{  first.PRAM.id
VAL [x.trans]INT first.PRAM.id IS [0,
   division.threshold[0] - 1,
   (division.threshold[0] +      division.threshold[1])  - 1,
   (division.threshold[0] + (2 * division.threshold[1])) - 1,

   (division.threshold[0] +  (3 * division.threshold[1])) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) +      division.threshold[4])) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (2 * division.threshold[4]))) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (3 * division.threshold[4]))) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (4 * division.threshold[4]))) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (5 * division.threshold[4]))) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (6 * division.threshold[4]))) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (7 * division.threshold[4]))) - 1,
   (division.threshold[0] + ((3 * division.threshold[1]) + (8 * division.threshold[4]))) - 1,

   (division.threshold[0] + ((3 * division.threshold[1]) + (9 * division.threshold[4]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) +       division.threshold[7])) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (2  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (3  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (4  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (5  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (6  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (7  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (8  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (9  * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (10 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (11 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (12 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (13 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (14 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (15 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (16 * division.threshold[7]))) - 1,
   (division.threshold[0] + (((3 * division.threshold[1]) + (9 * division.threshold[4])) + (17 * division.threshold[7]))) - 1] :

--{{{  COMMENT to first.PRAM.id
-- first.PRAM.id

-- For a transputer x (0 <= x < x.trans), first.PRAM.id[x] is an identifier
-- for the first PRAM processor simulated in that transputer. Such numbers
-- verify that they are sequenced from 0..N being N the total number of
-- PRAM processors available in the system.
--}}}  
--}}}  
--{{{  PRAM.descendents
VAL [x.trans]INT PRAM.descendents IS [(leaf.nodes * max.processor),
		 (6 * max.processor), (6 * max.processor), (6 * max.processor),
		 (2 * max.processor), (2 * max.processor), (2 * max.processor),
		 (2 * max.processor), (2 * max.processor), (2 * max.processor),
		 (2 * max.processor), (2 * max.processor), (2 * max.processor),
		 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0] :
--{{{  COMMENT to PRAM.descendents
-- PRAM.descendents

-- Is the number of PRAM processors situated below a considered transputer in the
-- transputer tree.
-- This value is used in the processor activation task: the activation won't be
-- carried on if the number of PRAM.processors is lower than the required number.
--}}}  
--}}}  
--{{{  G.initial.number
--                                   [0, 1, 2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30] :
VAL [x.trans]INT G.initial.number IS [1, 4, 7, 10, 13, 15, 17, 19, 21, 23, 25, 27, 29,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0] :
--{{{  COMMENT to G.initial.number
-- G.initial.number: This value is the number of the leftmost children
-- (the lower number) for each transputer.
--}}}  
--}}}  
--{{{  leaf
VAL [x.trans]BOOL leaf IS [FALSE,
		 FALSE, FALSE, FALSE,
		 FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE,
		 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE,
		 TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE] :

-- leaf[i] indicates if transputer i is or not a leaf node
-- in the tree structure.
--}}}  

-- #INCLUDE "errors.inc"  -- Error codes
